# -*- coding: utf-8 -*-
"""
    This module includes various helpers and fixers.
"""
from flask.json import JSONEncoder
from datetime import date


class CustomJSONEncoder(JSONEncoder):
    '''Extend the JSON encoder to treat date-time objects as strict
    rfc3339 types.
    '''
    def default(self, obj):
        from drift.orm import ModelBase
        if isinstance(obj, date):
            return obj.isoformat() + "Z"
        elif isinstance(obj, ModelBase):
            return obj.as_dict()
        else:
            return JSONEncoder.default(self, obj)


# Fixing SCRIPT_NAME/url_scheme when behind reverse proxy
# http://flask.pocoo.org/snippets/35/
class ReverseProxied(object):
    '''Wrap the application in this middleware and configure the
    front-end server to add these headers, to let you quietly bind
    this to a URL other than / and to an HTTP scheme that is
    different than what is used locally.

    In nginx:
    location /myprefix {
        proxy_pass http://192.168.0.1:5001;
        proxy_set_header Host $host;
        proxy_set_header X-Forwarded-For $proxy_add_x_forwarded_for;
        proxy_set_header X-Scheme $scheme;
        proxy_set_header X-Script-Name /myprefix;
        }

    :param app: the WSGI application
    '''
    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        script_name = environ.get('HTTP_X_SCRIPT_NAME', '')
        if script_name:
            environ['SCRIPT_NAME'] = script_name
            path_info = environ['PATH_INFO']
            if path_info.startswith(script_name):
                environ['PATH_INFO'] = path_info[len(script_name):]

        scheme = environ.get('HTTP_X_SCHEME', environ.get('X-Forwarded-Proto', ''))
        if scheme:
            environ['wsgi.url_scheme'] = scheme

        server = environ.get('HTTP_X_FORWARDED_SERVER', '')
        if server:
            environ['HTTP_HOST'] = server

        return self.app(environ, start_response)
